function [vmin_best, acbest, Bestcut]=hypergraphcut_V2(INC,w,vertex_weights,maxrun,init,p,verbosity)

% Balanced Hypergraph Cuts

% INC: We assume INC exists which is the hypergraph incidence matrix, every
% column represents a hypergraph edge (1 if the correspondent vertex given
% by the row number is in the edge and zero otherwise)

% w: the vector of edge weights

% maxrun: total number of initializations,

% if init is 1 then the first initialization is with the second eigenvector
% of the standard graph laplacian (zhou, sch??lkopf) otherwise all
% initializations are random

% createClustersUniversalHypergraph: computes the optimal clustering based
% on input vector vmin. Last input variable is unimportant (for hard
% balanced cut), second to last is always 1 since we look at l1 norm. Third
% to last is 1 for RCC/NCC and 2 for RCut/NCut. 4th to last is the vertex degree. 
%
% (C)2012-13 Matthias Hein, Simon Setzer, Leonardo Jost and Syama Sundar Rangapuram

[nVertices, nEdges] = size(INC);
%deg_normalized = sum(INC*spdiags(w,0,nEdges,nEdges),2);
Bestcut=inf;
Cuts=zeros(maxrun,1);
for run=1:maxrun
%     randinit=zeros(nVertices,1);
%     for i=1:nVertices
%         randinit(ceil(rand*nVertices))=1;
%     end
    %randinit=round(randn(nVertices,1));
    randinit=randn(nVertices,1);
    if (init==1 && run==1)
        randinit = power_norm_lapl_hypergraph(INC,w,10e-12);
    end
    %[vmin,RCC_NCC_Clusters,RCC_NCC] = ComputeEigCleanHypergraph(w,INC',randinit,1000,0.01,1,p);    
    if p==2
        %[vmin, RCC_NCC_Clusters, lambda, RCC_NCC]=ComputeEigCleanHypergraph_NCut(w,INC',INC*w,randinit,1000,1e-6, false, true);
        [vmin, RCC_NCC_Clusters, lambda, RCC_NCC]=ComputeEigCleanHypergraph_NCut(w,INC',vertex_weights,randinit,1000,1e-12, false, true,verbosity);
    else
        %[vmin, RCC_NCC_Clusters, RCC_NCC]=ComputeEigCleanHypergraph_Cheeger(w,INC',randinit,1000,1e-6);
        [vmin,RCC_NCC_Clusters,RCC_NCC] =ComputeEigCleanHypergraph(w,INC',randinit,1000,1e-6,true,p,verbosity);
    end
    % NCC
    %[RCC_NCC_Clusters, RCC_NCC,~,~] =  createClustersUniversalHypergraph(vmin,w,INC',deg_normalized,1,1,0);
    %fprintf('\n=> Hyp-IPM NCC: %f \n\n', RCC_NCC);
%     t=1;
%     for i=1:5
%         vnew=RCC_NCC_Clusters;
%         if rand>0.5
%             for j=1:nVertices
%                 if rand<0.05*t
%                     vnew(j)=1-vnew(j);
%                 end
%             end
%         else
%             vnew=vnew+0.05*t*randn(nVertices,1);
%         end
%         t=t+1;
%         [vmin_temp,RCC_NCC_Clusters_temp,RCC_NCC_temp] = ComputeEigCleanHypergraph(w,INC',vnew,10,0.01,1,p);
%         if RCC_NCC_temp<min(RCC_NCC)
%             RCC_NCC_Clusters=RCC_NCC_Clusters_temp;
%             %vmin=vmin_temp;
%             t=1;
%         end
%         RCC_NCC(i+1)=RCC_NCC_temp;
%     end
    Bestcut_temp=min(RCC_NCC);
    Cuts(run)=Bestcut_temp;
    if Bestcut_temp<Bestcut
        Bestcut=Bestcut_temp;
        acbest=RCC_NCC_Clusters;
        vmin_best = vmin;
    end
    %fprintf('perturbation=> Hyp-IPM NCC: %f \n', RCC_NCC);
end
%fprintf('Cuts: %f\n',Cuts);
%fprintf('Best Cut: %f\n',Bestcut);
